#include<linux/module.h>
#include<linux/random.h>
#include<linux/kfifo.h>

#define NUMBER_OF_ELEMENTS 8

static struct kfifo fifo_queue;

static int __init fifomod_init(void)
{
	int returned_value;
	u32 random_value;
	unsigned int returned_size;

	returned_value = kfifo_alloc(&fifo_queue,NUMBER_OF_ELEMENTS*sizeof(random_value),GFP_KERNEL);
	if(returned_value) {
		pr_alert("Error allocating kfifo!\n");
		return -ENOMEM;
	}
	while(!kfifo_is_full(&fifo_queue)) {
		get_random_bytes(&random_value,sizeof(random_value));
		random_value %= 100;
		returned_size = kfifo_in(&fifo_queue,&random_value,sizeof(random_value));
		if(returned_size!=sizeof(random_value))
			pr_alert("Enqueue error\n");
	}
	returned_size = kfifo_out_peek(&fifo_queue,&random_value,sizeof(random_value));
	if(returned_size!=sizeof(returned_size))
		pr_alert("Error peeking element form the queue!\n");
	pr_notice("Value of the first element in the queue: %u\n",random_value);

	return 0;
}

static void __exit fifomod_exit(void)
{
	int returned_size;
	u32 value;

	while(!kfifo_is_empty(&fifo_queue)) {
		returned_size = kfifo_out(&fifo_queue,&value,sizeof(value));
		if(returned_size!=sizeof(value))
			pr_alert("Dequeue error!\n");
		pr_notice("Value from the queue: %u\n",value);
	}

	kfifo_free(&fifo_queue);
}

module_init(fifomod_init);
module_exit(fifomod_exit);

MODULE_AUTHOR("Arkadiusz Chrobot <a.chrobot@tu.kielce.pl>");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("An exemplary module demonstrating the usage of a kernel FIFO queue.");
MODULE_VERSION("1.0");
